@@ -19,11 +19,11 @@
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
 # USA
 
-import signal
 import warnings
 import sys
 import socket
 
+from .._ossighelper import wakeup_on_signal, register_sigint_fallback
 from ..module import get_introspection_module
 from .._gi import (variant_type_from_string, source_new,
                    source_set_callback, io_channel_read)
@@ -40,15 +40,14 @@ __all__.append('option')
 
 
 # Types and functions still needed from static bindings
-from gi._gi import _glib
-from gi._gi import _gobject
+from gi import _gi
 from gi._error import GError
 
 Error = GError
-OptionContext = _glib.OptionContext
-OptionGroup = _glib.OptionGroup
-Pid = _glib.Pid
-spawn_async = _glib.spawn_async
+OptionContext = _gi.OptionContext
+OptionGroup = _gi.OptionGroup
+Pid = _gi.Pid
+spawn_async = _gi.spawn_async
 
 
 def threads_init():
@@ -251,7 +250,16 @@ class Variant(GLib.Variant):
         return GLib.Variant.new_tuple(elements)
 
     def __del__(self):
-        self.unref()
+        try:
+            self.unref()
+        except ImportError:
+            # Calling unref will cause gi and gi.repository.GLib to be
+            # imported. However, if the program is exiting, then these
+            # modules have likely been removed from sys.modules and will
+            # raise an exception. Assume that's the case for ImportError
+            # and ignore the exception since everything will be cleaned
+            # up, anyways.
+            pass
 
     def __str__(self):
         return self.print_(True)
@@ -468,6 +476,7 @@ def get_string(self):
     value, length = GLib.Variant.get_string(self)
     return value
 
+
 setattr(Variant, 'get_string', get_string)
 
 __all__.append('Variant')
@@ -478,6 +487,8 @@ def markup_escape_text(text, length=-1):
         return GLib.markup_escape_text(text.decode('UTF-8'), length)
     else:
         return GLib.markup_escape_text(text, length)
+
+
 __all__.append('markup_escape_text')
 
 
@@ -540,7 +551,7 @@ for name in ['G_MINFLOAT', 'G_MAXFLOAT', 'G_MINDOUBLE', 'G_MAXDOUBLE',
              'G_MAXUINT', 'G_MINLONG', 'G_MAXLONG', 'G_MAXULONG', 'G_MAXSIZE',
              'G_MINSSIZE', 'G_MAXSSIZE', 'G_MINOFFSET', 'G_MAXOFFSET']:
     attr = name.split("_", 1)[-1]
-    globals()[attr] = getattr(_gobject, name)
+    globals()[attr] = getattr(_gi, name)
     __all__.append(attr)
 
 
@@ -549,32 +560,14 @@ class MainLoop(GLib.MainLoop):
     def __new__(cls, context=None):
         return GLib.MainLoop.new(context, False)
 
-    # Retain classic pygobject behaviour of quitting main loops on SIGINT
     def __init__(self, context=None):
-        def _handler(loop):
-            loop.quit()
-            loop._quit_by_sigint = True
-            # We handle signal deletion in __del__, return True so GLib
-            # doesn't do the deletion for us.
-            return True
-
-        if sys.platform != 'win32':
-            # compatibility shim, keep around until we depend on glib 2.36
-            if hasattr(GLib, 'unix_signal_add'):
-                fn = GLib.unix_signal_add
-            else:
-                fn = GLib.unix_signal_add_full
-            self._signal_source = fn(GLib.PRIORITY_DEFAULT, signal.SIGINT, _handler, self)
-
-    def __del__(self):
-        if hasattr(self, '_signal_source'):
-            GLib.source_remove(self._signal_source)
+        pass
 
     def run(self):
-        super(MainLoop, self).run()
-        if hasattr(self, '_quit_by_sigint'):
-            # caught by _main_loop_sigint_handler()
-            raise KeyboardInterrupt
+        with register_sigint_fallback(self.quit):
+            with wakeup_on_signal():
+                super(MainLoop, self).run()
+
 
 MainLoop = override(MainLoop)
 __all__.append('MainLoop')
@@ -585,6 +578,7 @@ class MainContext(GLib.MainContext):
     def iteration(self, may_block=True):
         return super(MainContext, self).iteration(may_block)
 
+
 MainContext = override(MainContext)
 __all__.append('MainContext')
 
@@ -634,6 +628,7 @@ class Source(GLib.Source):
 
     can_recurse = property(__get_can_recurse, __set_can_recurse)
 
+
 Source = override(Source)
 __all__.append('Source')
 
@@ -649,6 +644,7 @@ class Idle(Source):
         if priority != GLib.PRIORITY_DEFAULT:
             self.set_priority(priority)
 
+
 __all__.append('Idle')
 
 
@@ -662,6 +658,7 @@ class Timeout(Source):
         if priority != GLib.PRIORITY_DEFAULT:
             self.set_priority(priority)
 
+
 __all__.append('Timeout')
 
 
@@ -670,6 +667,7 @@ def idle_add(function, *user_data, **kwargs):
     priority = kwargs.get('priority', GLib.PRIORITY_DEFAULT_IDLE)
     return GLib.idle_add(priority, function, *user_data)
 
+
 __all__.append('idle_add')
 
 
@@ -677,6 +675,7 @@ def timeout_add(interval, function, *user_data, **kwargs):
     priority = kwargs.get('priority', GLib.PRIORITY_DEFAULT)
     return GLib.timeout_add(priority, interval, function, *user_data)
 
+
 __all__.append('timeout_add')
 
 
@@ -684,6 +683,7 @@ def timeout_add_seconds(interval, function, *user_data, **kwargs):
     priority = kwargs.get('priority', GLib.PRIORITY_DEFAULT)
     return GLib.timeout_add_seconds(priority, interval, function, *user_data)
 
+
 __all__.append('timeout_add_seconds')
 
 
@@ -741,6 +741,7 @@ def _io_add_watch_get_args(channel, priority_, condition, *cb_and_user_data, **k
 
     return real_channel, priority_, condition, func_fdtransform, user_data
 
+
 __all__.append('_io_add_watch_get_args')
 
 
@@ -749,6 +750,7 @@ def io_add_watch(*args, **kwargs):
     channel, priority, condition, func, user_data = _io_add_watch_get_args(*args, **kwargs)
     return GLib.io_add_watch(channel, priority, condition, func, *user_data)
 
+
 __all__.append('io_add_watch')
 
 
@@ -830,6 +832,7 @@ class IOChannel(GLib.IOChannel):
     # Python 2.x compatibility
     next = __next__
 
+
 IOChannel = override(IOChannel)
 __all__.append('IOChannel')
 
@@ -844,6 +847,7 @@ class PollFD(GLib.PollFD):
         self.fd = fd
         self.events = events
 
+
 PollFD = override(PollFD)
 __all__.append('PollFD')
 
@@ -891,6 +895,7 @@ def _child_watch_add_get_args(priority_or_pid, pid_or_callback, *args, **kwargs)
 
     return priority, pid, callback, user_data
 
+
 # we need this to be accessible for unit testing
 __all__.append('_child_watch_add_get_args')
 
@@ -900,12 +905,14 @@ def child_watch_add(*args, **kwargs):
     priority, pid, function, data = _child_watch_add_get_args(*args, **kwargs)
     return GLib.child_watch_add(priority, pid, function, *data)
 
+
 __all__.append('child_watch_add')
 
 
 def get_current_time():
     return GLib.get_real_time() * 0.000001
 
+
 get_current_time = deprecated(get_current_time, 'GLib.get_real_time()')
 
 __all__.append('get_current_time')
@@ -916,6 +923,7 @@ __all__.append('get_current_time')
 def filename_from_utf8(utf8string, len=-1):
     return GLib.filename_from_utf8(utf8string, len)[0]
 
+
 __all__.append('filename_from_utf8')
 
 
